% This script illustrates function handles and function functions
%
clc
disp('         Function Handles and Function Functions')
disp(' ')
disp('The name of an anonymous function is known as a function handle.')
disp('We can define')
disp('>> sqr = @(x) x.^2')
sqr = @(x) x.^2
disp('Paused...Please hit a key to continue')
pause
disp('Note that this function will operate elementwise on vectors or matrices.')
disp('For example')
disp('>> sqr(5)')
sqr(5)
disp('Paused...Please hit a key to continue')
pause
disp('or')
disp('>> sqr(1:5)')
sqr(1:5)
disp('Paused...Please hit a key to continue')
pause
disp('For example')
disp('>> a = floor(10*rand(2,2))')
disp('>> a_squared = sqr(a)')
a = floor(10*rand(2,2))
a_squared = sqr(a)
disp('Paused...Please hit a key to continue')
pause
disp(' ')
disp('MATLAB includes many so-called ''function functions''.')
disp('These are functions that take function handles as input.')
disp('Examples are ''quad'' (for numerical integration),')
disp('certain calculus-related routines (such as ''fminbnd'' or ''fzero''),')
disp('and many commands for the solution of differential equations')
disp('such as ''ode45'' or ''ode15s''.')
disp('Of course, you can also write your own function functions.')
disp('We now give three examples.')
disp(' ')
disp('Numerical Integration:')
disp('To integrate the function f(x)=x^2 on the interval [0,1]')
disp('we can pass the function handle defined above to the ''quad'' command')
disp('>> quad(sqr,0,1)')
quad(sqr,0,1)
disp('Paused...Please hit a key to continue')
pause
disp(' ')
disp('Minimization:')
disp('To find the minimum of the function f(x)=(x-2)^2-5 on the interval [-3,3]')
disp('we can define a new function handle and then pass it to ''fminbnd''.')
disp('>> sqr2 = @(x) (x-2).^2-5')
sqr2 = @(x) (x-2).^2-5
disp('The minimum occurs at')
disp('>> fminbnd(sqr2,-3,3)')
fminbnd(sqr2,-3,3)
disp('and its value is')
disp('>> sqr2(ans)')
sqr2(ans)
disp('Paused...Please hit a key to continue')
pause
disp(' ')
disp('Find zeros:')
disp('To find the zeros of the function f(x)=(x-2)^2-5 on the interval [-3,3]')
disp('we can use the function handle just defined together with ''fzero''.')
disp('>> fzero(sqr2,-3,3)')
fzero(sqr2,-3,3)
disp('Paused...Please hit a key to continue')
pause
disp('Here is a plot of this function')
disp('>> ezplot(sqr2,[-3,3]); grid on')
ezplot(sqr2,[-3,3]); grid on
disp('Paused...Please hit a key to continue')
pause
disp('or - using the standard plot command:')
disp('>> x = linspace(-3,3,100);')
x = linspace(-3,3,100);
disp('>> figure; plot(x,sqr2(x)); grid on')
figure; plot(x,sqr2(x)); grid on
